@@ -377,6 +377,7 @@ int cw_mp3_handler( int ops, _main_data *main_data, GtkWidget *notebook )
 
 	static struct {
 		GtkWidget *menu_item;
+		int encoding_type;
 		char *encoder;
 		char *plugin;
 		char *description;
@@ -386,17 +387,17 @@ int cw_mp3_handler( int ops, _main_data *main_data, GtkWidget *notebook )
 		char *high_qual_op;
 		char *crc_op;
 	} plugins[] = {
- 		{ NULL, "toolame", "ripperX_plugin-toolame", "toolame layer 2 encoder", "-b", "", "-v", "", "-e"},
- 		{ NULL, "encode ", "ripperX_plugin-encode", "ISO Encoder v2", "-b", "", "", "", "-e"},
- 		{ NULL, "8hz-mp3 ", "ripperX_plugin-8hz-mp3", "8hz-mp3 Encoder", "-b", "", "", "", ""},
- 		{ NULL, "lame", "ripperX_plugin-lame", "Lame MP3 Encoder", "-b", "--nohist -v", "-V", "-h", "-p"},
- 		{ NULL, "gogo", "ripperX_plugin-gogo", "GoGo MP3 Encoder", "-b", "-v 4", "", "", ""},
- 		{ NULL, "bladeenc", "ripperX_plugin-bladeenc", "BladeEnc MP3 Encoder", "-br", "", "", "", "-crc"},
- 		{ NULL, "xingmp3enc", "ripperX_plugin-xingmp3enc", "Xingmp3enc MP3 Encoder", "-B", "", "", "", ""},
- 		{ NULL, "l3enc", "ripperX_plugin-l3enc", "FHG MP3 Encoder (l3enc v2.72)", "-br", "", "", "-hq", "-crc"},
- 		{ NULL, "mp3enc", "ripperX_plugin-mp3enc", "FHG MP3 Encoder (mp3enc 3.1)", "-br", "", "", "-qual 9", "-crc"},
- 		{ NULL, "oggenc", "ripperX_plugin-oggenc", "OggVorbis encoder", "-b", "", "", "", "" },
- 		{ NULL, "flac", "ripperX_plugin-flac", "FLAC encoder", "", "", "", "", "" }
+ 		{ NULL, MP3, "toolame", "ripperX_plugin-toolame", "toolame layer 2 encoder", "-b", "", "-v", "", "-e"},
+ 		{ NULL, MP3, "encode ", "ripperX_plugin-encode", "ISO Encoder v2", "-b", "", "", "", "-e"},
+ 		{ NULL, MP3, "8hz-mp3 ", "ripperX_plugin-8hz-mp3", "8hz-mp3 Encoder", "-b", "", "", "", ""},
+ 		{ NULL, MP3, "lame", "ripperX_plugin-lame", "Lame MP3 Encoder", "-b", "--nohist -v", "-V", "-h", "-p"},
+ 		{ NULL, MP3, "gogo", "ripperX_plugin-gogo", "GoGo MP3 Encoder", "-b", "-v 4", "", "", ""},
+ 		{ NULL, MP3, "bladeenc", "ripperX_plugin-bladeenc", "BladeEnc MP3 Encoder", "-br", "", "", "", "-crc"},
+ 		{ NULL, MP3, "xingmp3enc", "ripperX_plugin-xingmp3enc", "Xingmp3enc MP3 Encoder", "-B", "", "", "", ""},
+ 		{ NULL, MP3, "l3enc", "ripperX_plugin-l3enc", "FHG MP3 Encoder (l3enc v2.72)", "-br", "", "", "-hq", "-crc"},
+ 		{ NULL, MP3, "mp3enc", "ripperX_plugin-mp3enc", "FHG MP3 Encoder (mp3enc 3.1)", "-br", "", "", "-qual 9", "-crc"},
+ 		{ NULL, OGG, "oggenc", "ripperX_plugin-oggenc", "OggVorbis encoder", "-b", "", "", "", "" },
+ 		{ NULL, FLAC, "flac", "ripperX_plugin-flac", "FLAC encoder", "", "", "", "", "" }
 	};
 
 	static struct {
@@ -578,6 +579,7 @@ int cw_mp3_handler( int ops, _main_data *main_data, GtkWidget *notebook )
 			}
 			strcpy( config.encoder.encoder, plugins[ cur_plugin ].encoder );
 			strcpy( config.encoder.plugin, plugins[ cur_plugin ].plugin );
+			main_data->encoding_type = plugins[ cur_plugin ].encoding_type;
 
 			if ( GTK_TOGGLE_BUTTON( use_varbitrate_ckbx ) ->active )
 				config.encoder.use_varbitrate = 1;
@@ -670,7 +672,7 @@ int cw_mp3_handler( int ops, _main_data *main_data, GtkWidget *notebook )
 			          config.encoder.use_crc ? plugins[ cur_plugin ].crc_op : "",
 					  config.encoder.use_varbitrate ? vbr_qual_switch : "",
 			          config.encoder.extra_options );
-                        }
+            }
 			return 0;
 		}
 	}
