@@ -67,6 +67,6 @@ extern int	id3_replacetag ( FILE *fout, id3tag_t *id3tag );
 extern void	id3_cleartag( id3tag_t *id3tag );
 extern char	*id3_findstyle( int styleid );
 unsigned char id3_find_cddb_category( char *name );
-void vorbistag(char *ogg_file, id3tag_t *id3tag); 
+void vorbistag(char *ogg_file, char *artist, char *album, char *title, unsigned char style, unsigned char track); 
 
 #endif     /* #ifndef _ID3_H */
