@@ -1,5 +1,5 @@
 /* file job_control.c
- * 
+ *
  * does calling of the ripper or encoder
  * handles files
  *
@@ -7,6 +7,10 @@
  */
 
 #include <stdlib.h>
+
+
+#include <stdio.h>		/* for bezo debug */
+
 #include <errno.h>            /* for errno */
 #include <sys/stat.h>         /* defines read and write flags */
 #include <time.h>
@@ -20,6 +24,8 @@
 #include <string.h>
 #include <gtk/gtk.h>
 
+#include <id3.h>
+
 #include "misc_utils.h"
 #include "interface_common.h"
 #include "id3.h"
@@ -51,6 +57,7 @@ typedef struct {
 	char *msg;
 } _dialog_data;
 
+
 /* Function Prototypes */
 int lock_file( char *file_name, int is_temp );
 
@@ -187,9 +194,7 @@ void calc_stat( _main_data *main_data, _stat *stat,
 			stat->wav_time_elapsed = cur_time - wav_start_time;
 			wav_length_processed = current;
 
-			stat->wav_progress = ( float ) wav_length_processed
-			                     / main_data->track[ cur_track ].length;
-
+			stat->wav_progress = ( float ) wav_length_processed / main_data->track[ cur_track ].length;
 			wav_temp_ratio = ( float ) ( stat->wav_time_elapsed ) / wav_length_processed;
 
 			if ( count++ >= COUNT_BEFORE_GET_AVG ) {
@@ -210,8 +215,7 @@ void calc_stat( _main_data *main_data, _stat *stat,
 			stat->mp3_time_elapsed = cur_time - mp3_start_time;
 			mp3_length_processed = current;
 
-			stat->mp3_progress = ( float ) mp3_length_processed
-			                     / main_data->track[ cur_track ].length;
+			stat->mp3_progress = ( float ) mp3_length_processed / main_data->track[ cur_track ].length;
 
 			mp3_temp_ratio = ( float ) ( stat->mp3_time_elapsed ) / mp3_length_processed;
 
@@ -418,8 +422,12 @@ void job_finisher( _main_data *main_data )
 	int madewavs = FALSE;
 	int mademp3s = FALSE;
 	int tracksdone = 0;
+	char s_track_num[2];
 	char *artist;
 
+	FILE *fp_playlist = NULL;
+	char playlist_filespec[ MAX_FILE_PATH_LENGTH + MAX_FILE_NAME_LENGTH ];
+
 	buffer[0] = 0;
 
 	/* Clean up */
@@ -434,47 +442,70 @@ void job_finisher( _main_data *main_data )
 		}
 		main_data->track[ i ].make_wav = FALSE;
 
-		if ( main_data->track[ i ].mp3_exist == TRUE &&
-			 main_data->track[ i ].make_mp3 == TRUE )
+		if ( main_data->track[ i ].mp3_exist == TRUE && main_data->track[ i ].make_mp3 == TRUE )
 		{
 			mademp3s = TRUE;
-/*			strcat( status_message, enc_file );	*/
-/*			strcat( status_message, "\n" );					*/
 
 			/* add ID3 tag if requested */
 			if ( config.cddb_config.create_id3 == TRUE ) {
-				id3tag_t mytag;
-
-				id3_cleartag( &mytag );
-				strncpy( mytag.songname, main_data->track[ i ].title, 30 );
 				if ( !(artist = main_data->track[ i ].artist) )
 					artist = main_data->disc_artist;
-				strncpy( mytag.artist, artist, 30 );
-				strncpy( mytag.album, main_data->disc_title, 30 );
- 				mytag.track = i + 1;
-				mytag.style = id3_find_cddb_category( main_data->disc_category );
 
-                if (strstr( enc_file, ".ogg" ) == NULL) {
-                    FILE *fptr = NULL;
-
-                    fptr = fopen( enc_file, "ab" );
-                    if ( fptr != NULL ) {
-                        id3_appendtag( fptr, &mytag );
-                        fclose( fptr );
-                    }
-                } else {
-                    vorbistag(enc_file, &mytag);
-                }
-	
+				// TODO:  fix this to use something like
+				//		if (main_data->encoding_type == OGG) {
+					
+				if ( !strcmp( config.encoder.encoder, "oggenc") ) {
+					// TODO:  add year to OGG tag
+					// strncpy( mytag.year, main_data->disc_year, 4 );	// M.Tyler 6-3-2005
+					vorbistag(enc_file, 
+							artist, 
+							main_data->disc_title, 
+							main_data->track[i].title,
+							id3_find_cddb_category( main_data->disc_category ),
+							i + 1 );
+				} else if ( !strcmp( config.encoder.encoder, "flac" ) ) {
+					/* nothing to do for FLAC right now */
+				} else {
+					/* assume MP3 tag is desired */
+					sprintf(s_track_num,"%d",(i+1));
+				
+					ID3Tag *myTag;
+					myTag=ID3Tag_New();
+					ID3Tag_Link(myTag,enc_file);
+				
+					set_TagField(myTag, main_data->track[ i ].title, ID3FID_TITLE);
+					set_TagField(myTag, artist, ID3FID_LEADARTIST);
+					set_TagField(myTag, main_data->disc_title, ID3FID_ALBUM);
+					set_TagField(myTag, main_data->disc_year, ID3FID_YEAR);
+					set_TagField(myTag, s_track_num, ID3FID_TRACKNUM);
+					set_TagField(myTag, main_data->disc_category, ID3FID_CONTENTTYPE);
+					ID3Tag_UpdateByTagType(myTag, ID3TT_ID3V2);
+					ID3Tag_Delete(myTag);
+				}
 			}
-			
+
 			//dc: strcat() is for sissies!
-	    		sprintf(&buffer[strlen(buffer)], "%d: %s\n",
-				++tracksdone, file_name_without_path(enc_file));
-		       			
+			sprintf(&buffer[strlen(buffer)], "%d: %s\n", ++tracksdone, file_name_without_path(enc_file));
+
+			// tm: basic playlist support - thanks to Mark Tyler
+			if ( config.cddb_config.create_playlist == TRUE )
+			{
+				if ( fp_playlist == NULL )
+				{
+					sprintf(playlist_filespec, "%s/playlist.m3u", file_path_without_name(enc_file));
+					fp_playlist = fopen(playlist_filespec, "w");
+				}
+
+				// if we succeeded above, we can now write to this
+				if ( fp_playlist != NULL )
+					fprintf( fp_playlist, "%s\n", file_name_without_path(enc_file));
+			}
 		}
 		main_data->track[ i ].make_mp3 = FALSE;
-	}
+	} /* end loop over all tracks */
+
+	if (( config.cddb_config.create_playlist == TRUE ) && ( fp_playlist != NULL ))
+		fclose( fp_playlist );
 
 	/* Generate status message */
 	sprintf(status_message, "Tracks Completed: %2d\n\nArtist: %s\nAlbum: %s\n\n%s",
@@ -585,430 +616,416 @@ void job_controller( int ops, _main_data *main_data )
 	int temp, temp_track;
 
 	switch ( ops ) {
-	case JC_START :
-		/* called once when the go button is clicked. */
-		saved_main_data = main_data;
+		case JC_START :
+			/* called once when the go button is clicked. */
+			saved_main_data = main_data;
 
-		if ( wav_cur_track != -1 ) {
-			err_handler( JOB_IN_PROGRESS_ERR, NULL );
-			job_finisher( main_data );
+			if ( wav_cur_track != -1 ) {
+				err_handler( JOB_IN_PROGRESS_ERR, NULL );
+				job_finisher( main_data );
+				return;
+			}
+
+			if ( find_next_job( main_data, wav_cur_track, WAV,
+						&wav_cur_track, &cur_type ) < 0 ) {
+				/* ok, no more wavs to rip, try looking for mp3s */
+				if ( find_next_job( main_data, mp3_cur_track, MP3,
+							&mp3_cur_track, &cur_type ) < 0 ) {
+					/* nothing at all found */
+					err_handler( NOTHING_TO_DO_ERR, NULL );
+					job_finisher( main_data );
+					return;
+				}
+				create_file_names_for_track(main_data, mp3_cur_track,
+						&wav_file_path, &enc_file_path);
+
+				calc_stat( main_data, &stat, mp3_current,
+						mp3_cur_track, MP3, CALC_START_SESSION );
+				calc_stat( main_data, &stat, mp3_current,
+						mp3_cur_track, MP3, CALC_START );
+
+				/* start encoding */
+				if ( start_ripping_encoding( MP3, main_data->track[ mp3_cur_track ].begin,
+							main_data->track[ mp3_cur_track ].length,
+							mp3_cur_track,
+							wav_file_path,
+							enc_file_path,
+							&mp3_pg_pid,
+							&mp3_pi_pid,
+							&mp3_read_fd ) < 0 )
+				{
+					job_finisher( main_data );
+					return;
+				}
+				stat.encoding = TRUE;
+			} else {
+				/* found the first wav to rip */
+				create_file_names_for_track(main_data, wav_cur_track, &wav_file_path, &enc_file_path);
+				calc_stat( main_data, &stat, wav_current, wav_cur_track, WAV, CALC_START_SESSION );
+				calc_stat( main_data, &stat, wav_current, wav_cur_track, WAV, CALC_START );
+
+				/* start ripping */
+				if ( start_ripping_encoding( WAV, main_data->track[ wav_cur_track ].begin,
+							main_data->track[ wav_cur_track ].length,
+							wav_cur_track,
+							wav_file_path,
+							enc_file_path,
+							&wav_pg_pid,
+							&wav_pi_pid,
+							&wav_read_fd ) < 0 )
+				{
+					job_finisher( main_data );
+					return;
+				}
+				stat.ripping = TRUE;
+
+				/* Find the next track to encode if any */
+				if (find_next_job( main_data, mp3_cur_track, MP3, &mp3_nxt_track, &cur_type) < 0 ||
+						!main_data->track[ mp3_nxt_track ].make_mp3)
+				{
+					mp3_nxt_track = -1;
+				}
+			}
+
+			/* Create wav/mp3 status frame */
+			wm_status_frame_handler( WIDGET_CREATE, cur_type, &stat, NULL );
+
+			job_controller_timeout_start();
 			return;
-		}
 
-		if ( find_next_job( main_data, wav_cur_track, WAV,
-			                    &wav_cur_track, &cur_type ) < 0 ) {
-			/* ok, no more wavs to rip, try looking for mp3s */
-			if ( find_next_job( main_data, mp3_cur_track, MP3,
-				                    &mp3_cur_track, &cur_type ) < 0 ) {
-				/* nothing at all found */
-				err_handler( NOTHING_TO_DO_ERR, NULL );
-				job_finisher( main_data );
-				return;
-			}
-			create_file_names_for_track(main_data, mp3_cur_track,
-										&wav_file_path, &enc_file_path);
+		case JC_UPDATE :
+			main_data = saved_main_data;
 
-			calc_stat( main_data, &stat, mp3_current,
-			           mp3_cur_track, MP3, CALC_START_SESSION );
-			calc_stat( main_data, &stat, mp3_current,
-			           mp3_cur_track, MP3, CALC_START );
+			/* Part 1: check progress on the rip */
+			if ( stat.ripping == TRUE ) {
+				temp = read_and_process_plugin_output( wav_read_fd, &wav_progress, msg );
+				wav_current = main_data->track[ wav_cur_track ].length * wav_progress;
 
-			/* start encoding */
-			if ( start_ripping_encoding( MP3,
-				                             main_data->track[ mp3_cur_track ].begin,
-				                             main_data->track[ mp3_cur_track ].length,
-				                             mp3_cur_track,
-				                             wav_file_path,
-				                             enc_file_path,
-				                             &mp3_pg_pid, &mp3_pi_pid,
-				                             &mp3_read_fd ) < 0 ) {
-				job_finisher( main_data );
-				return;
-			}
-			stat.encoding = TRUE;
-		} else {
-			/* found the first wav to rip */
-			create_file_names_for_track(main_data, wav_cur_track,
-										&wav_file_path, &enc_file_path);
-			calc_stat( main_data, &stat, wav_current,
-			           wav_cur_track, WAV, CALC_START_SESSION );
-			calc_stat( main_data, &stat, wav_current,
-			           wav_cur_track, WAV, CALC_START );
+				switch ( temp ) {
+					case PLUGIN_MSG_PARSE_ERR :
+						/* Nothing to do. Let's wait more */
+						break;
 
-			/* start ripping */
-			if ( start_ripping_encoding( WAV,
-				                             main_data->track[ wav_cur_track ].begin,
-				                             main_data->track[ wav_cur_track ].length,
-				                             wav_cur_track,
-				                             wav_file_path,
-				                             enc_file_path,
-				                             &wav_pg_pid, &wav_pi_pid,
-				                             &wav_read_fd ) < 0 ) {
-				job_finisher( main_data );
-				return;
-			}
-			stat.ripping = TRUE;
+					case PLUGIN_PROGRESS_MSG :
+						/* progress report, update status display */
+						calc_stat( main_data, &stat, wav_current,
+								wav_cur_track, WAV, CALC_UPDATE );
+						/* Update status widget */
+						if ( msg[ 0 ] == '\0' )
+							str = NULL;
+						else
+							str = msg;
+						wm_status_frame_handler( WIDGET_UPDATE, WAV, &stat, str );
+						break;
 
-			/* Find the next track to encode if any */
-			if (find_next_job( main_data, mp3_cur_track, MP3, &mp3_nxt_track, &cur_type) < 0 ||
-				!main_data->track[ mp3_nxt_track ].make_mp3)
-			{
-				mp3_nxt_track = -1;
-			}
-		}
+					case PLUGIN_NO_MSG_AVAILABLE :
+						/* Check if the plugin has exited
+						   It only happens when ripperX failed to execute the plugin */
+						if ( wav_pi_pid >= 0 )
+							if ( waitpid( wav_pi_pid, NULL, WNOHANG ) == wav_pi_pid ) {
+								err_handler( PLUGIN_NOT_PRESENT_ERR,
+										"Maybe ripperX has failed to execute the plugin" );
+								wav_pi_pid = -1;
+								job_controller_timeout_stop();
+								job_controller( JC_ABORT_ALL_DELETE, main_data );
+								return;
+							}
 
-		/* Create wav/mp3 status frame */
-		wm_status_frame_handler( WIDGET_CREATE, cur_type, &stat, NULL );
+						/* Check if the job is finished */
+						if ( waitpid( wav_pg_pid, NULL, WNOHANG ) == wav_pg_pid ) {
+							/* One job finished, go for next one */
 
-		job_controller_timeout_start();
-		return;
+							/* kill the plugin */
+							if ( waitpid( wav_pi_pid, NULL, WNOHANG ) != wav_pi_pid ) {
+								kill( wav_pi_pid, SIGTERM );
+								waitpid( wav_pi_pid, NULL, 0 );
+							}
 
-	case JC_UPDATE :
-		main_data = saved_main_data;
+							/* Close the fd */
+							close( wav_read_fd );
 
-		/* Part 1: check progress on the rip */
-		if ( stat.ripping == TRUE ) {
-			temp = read_and_process_plugin_output( wav_read_fd, &wav_progress, msg );
-			wav_current = main_data->track[ wav_cur_track ].length * wav_progress;
-
-			switch ( temp ) {
-			case PLUGIN_MSG_PARSE_ERR :
-				/* Nothing to do. Let's wait more */
-				break;
-
-			case PLUGIN_PROGRESS_MSG :
-				/* progress report, update status display */
-				calc_stat( main_data, &stat, wav_current,
-				           wav_cur_track, WAV, CALC_UPDATE );
-				/* Update status widget */
-				if ( msg[ 0 ] == '\0' )
-					str = NULL;
-				else
-					str = msg;
-				wm_status_frame_handler( WIDGET_UPDATE, WAV, &stat, str );
-				break;
-
-			case PLUGIN_NO_MSG_AVAILABLE :
-				/* Check if the plugin has exited
-				 It only happens when ripperX failed to execute the plugin */
-				if ( wav_pi_pid >= 0 )
-					if ( waitpid( wav_pi_pid, NULL, WNOHANG ) == wav_pi_pid ) {
-						err_handler( PLUGIN_NOT_PRESENT_ERR,
-						             "Maybe ripperX has failed to execute the plugin" );
-						wav_pi_pid = -1;
-						job_controller_timeout_stop();
-						job_controller( JC_ABORT_ALL_DELETE, main_data );
-						return;
-					}
-
-				/* Check if the job is finished */
-				if ( waitpid( wav_pg_pid, NULL, WNOHANG ) == wav_pg_pid ) {
-					/* One job finished, go for next one */
-
-					/* kill the plugin */
-					if ( waitpid( wav_pi_pid, NULL, WNOHANG ) != wav_pi_pid ) {
-						kill( wav_pi_pid, SIGTERM );
-						waitpid( wav_pi_pid, NULL, 0 );
-					}
-
-					/* Close the fd */
-					close( wav_read_fd );
-
-					/* stop calculating stats */
-					calc_stat( main_data, &stat, wav_current,
-					           wav_cur_track, WAV, CALC_STOP );
-
-					/* Mark that it exists */
-					main_data->track[ wav_cur_track ].wav_exist = TRUE;
-
-					/* find next track to rip */
-					temp_track = wav_cur_track;
-					if ( find_next_job( main_data, wav_cur_track, WAV, &wav_cur_track, &cur_type ) < 0 ) {
-						/* All finished - no more rips */
-						wav_pg_pid = -1;
-						wav_pi_pid = -1;
-						wav_cur_track = -1;
-						cur_type = -1;
-						stat.ripping = FALSE;
-
-						/* if we are only ripping, finish up for good */
-						if ( !main_data->track[ temp_track ].make_mp3 ) {
+							/* stop calculating stats */
 							calc_stat( main_data, &stat, wav_current,
-							           temp_track, WAV, CALC_STOP_SESSION );
+									wav_cur_track, WAV, CALC_STOP );
+
+							/* Mark that it exists */
+							main_data->track[ wav_cur_track ].wav_exist = TRUE;
+
+							/* find next track to rip */
+							temp_track = wav_cur_track;
+							if ( find_next_job( main_data, wav_cur_track, WAV, &wav_cur_track, &cur_type ) < 0 ) {
+								/* All finished - no more rips */
+								wav_pg_pid = -1;
+								wav_pi_pid = -1;
+								wav_cur_track = -1;
+								cur_type = -1;
+								stat.ripping = FALSE;
+
+								/* if we are only ripping, finish up for good */
+								if ( !main_data->track[ temp_track ].make_mp3 ) {
+									calc_stat( main_data, &stat, wav_current,
+											temp_track, WAV, CALC_STOP_SESSION );
+
+									job_controller_timeout_stop();
+									job_finisher( main_data );
+									return;
+								}
+							} else {
+								/* if we are only ripping, update the stats */
+								if ( !main_data->track[ temp_track ].make_mp3 ) {
+									stat.tracks_done++;
+									stat.tracks_remain--;
+								}
+
+								/*  start ripping the next track */
+								create_file_names_for_track(main_data, wav_cur_track,
+										&wav_file_path, &enc_file_path);
+								calc_stat( main_data, &stat, wav_current,
+										wav_cur_track, WAV, CALC_START );
+								if ( start_ripping_encoding( WAV,
+											main_data->track[ wav_cur_track ].begin,
+											main_data->track[ wav_cur_track ].length,
+											wav_cur_track,
+											wav_file_path,
+											enc_file_path,
+											&wav_pg_pid, &wav_pi_pid,
+											&wav_read_fd ) < 0 ) {
+									calc_stat( main_data, &stat, wav_current,
+											wav_cur_track, WAV, CALC_STOP_SESSION );
+									job_controller_timeout_stop();
+									job_finisher( main_data );
+									return;
+								}
+							}
+						} /* end if job is finished section */
+						break;
+				} /* end rip switch */
+			} /* end rip progress check */
+
+			/* Part 2: check progress on the encode */
+			if ( stat.encoding == TRUE ) {
+				temp = read_and_process_plugin_output( mp3_read_fd, &mp3_progress, msg );
+				mp3_current = main_data->track[ mp3_cur_track ].length * mp3_progress;
+
+				switch ( temp ) {
+					case PLUGIN_MSG_PARSE_ERR :
+						/* Nothing to do. Let's wait more */
+						break;
+
+					case PLUGIN_PROGRESS_MSG :
+						/* progress report, update status display */
+						calc_stat( main_data, &stat, mp3_current,
+								mp3_cur_track, MP3, CALC_UPDATE );
+						/* Update status widget */
+						if ( msg[ 0 ] == '\0' )
+							str = NULL;
+						else
+							str = msg;
+						wm_status_frame_handler( WIDGET_UPDATE, MP3, &stat, str );
+						break;
+
+					case PLUGIN_NO_MSG_AVAILABLE :
+						/* Check if the plugin has exited
+						   It only happens when ripperX failed to execute the plugin */
+						if ( mp3_pi_pid >= 0 )
+							if ( waitpid( mp3_pi_pid, NULL, WNOHANG ) == mp3_pi_pid ) {
+								err_handler( PLUGIN_NOT_PRESENT_ERR,
+										"Maybe ripperX has failed to execute the plugin" );
+								mp3_pi_pid = -1;
+								job_controller_timeout_stop();
+								job_controller( JC_ABORT_ALL_DELETE, main_data );
+								return;
+							}
+
+						/* Check if the job is finished */
+						if ( waitpid( mp3_pg_pid, NULL, WNOHANG ) == mp3_pg_pid ) {
+							/* One job finished, go for next one */
+
+							/* kill the plugin */
+							if ( waitpid( mp3_pi_pid, NULL, WNOHANG ) != mp3_pi_pid ) {
+								kill( mp3_pi_pid, SIGTERM );
+								waitpid( mp3_pi_pid, NULL, 0 );
+							}
+
+							/* Close the fd */
+							close( mp3_read_fd );
+
+							/* stop calculating stats */
+							calc_stat( main_data, &stat, mp3_current,
+									mp3_cur_track, MP3, CALC_STOP );
+
+							main_data->track[ mp3_cur_track ].mp3_exist = TRUE;
+							/* Delete WAV file if he/she doesn't want it */
+							if ( !config.keep_wav ) {
+								create_file_names_for_track(main_data, mp3_cur_track,
+										&wav_file_path, &enc_file_path);
+								if ( unlink( wav_file_path ) < 0 )
+									err_handler( FILE_DELETE_ERR, wav_file_path );
+								/* Mark that it has been deleted */
+								main_data->track[ mp3_cur_track ].wav_exist = FALSE;
+
+								/* Delete WAV work directory if this was the last WAV file,
+								   and the mp3 work dir is different */
+								if (stat.tracks_remain <= 1)
+									if (strcmp(config.wav_path,config.mp3_path) != 0)
+										rmdir(file_path_without_name(wav_file_path));
+							}
+
+							/* find next track to encode */
+							temp_track = mp3_cur_track;
+							if ( find_next_job( main_data, mp3_cur_track, MP3, &mp3_nxt_track, &cur_type ) < 0 ) {
+								/* All finished - no more encoding - done for good! */
+								mp3_pg_pid = -1;
+								mp3_pi_pid = -1;
+								mp3_cur_track = mp3_nxt_track = -1;
+								cur_type = -1;
+								calc_stat( main_data, &stat, mp3_current,
+										temp_track, MP3, CALC_STOP_SESSION );
+								job_controller_timeout_stop();
+								stat.encoding = FALSE;
+
+								job_finisher( main_data );
+								return;
+							}
+							mp3_cur_track = -1;
 
-							job_controller_timeout_stop();
-							job_finisher( main_data );
-							return;
-						}
-					} else {
-						/* if we are only ripping, update the stats */
-						if ( !main_data->track[ temp_track ].make_mp3 ) {
 							stat.tracks_done++;
 							stat.tracks_remain--;
-						}
+							stat.encoding = FALSE;
 
-						/*  start ripping the next track */
-						create_file_names_for_track(main_data, wav_cur_track,
-													&wav_file_path, &enc_file_path);
-						calc_stat( main_data, &stat, wav_current,
-						           wav_cur_track, WAV, CALC_START );
-						if ( start_ripping_encoding( WAV,
-							                             main_data->track[ wav_cur_track ].begin,
-							                             main_data->track[ wav_cur_track ].length,
-							                             wav_cur_track,
-														 wav_file_path,
-														 enc_file_path,
-							                             &wav_pg_pid, &wav_pi_pid,
-							                             &wav_read_fd ) < 0 ) {
-							calc_stat( main_data, &stat, wav_current,
-							           wav_cur_track, WAV, CALC_STOP_SESSION );
-							job_controller_timeout_stop();
-							job_finisher( main_data );
-							return;
-						}
-					}
-				} /* end if job is finished section */
-				break;
-			} /* end rip switch */
-		} /* end rip progress check */
+						} /* end if job is finished section */
+						break;
+				} /* end encode switch */
+			} /* end encode progress check */
 
-		/* Part 2: check progress on the encode */
-		if ( stat.encoding == TRUE ) {
-			temp = read_and_process_plugin_output( mp3_read_fd, &mp3_progress, msg );
-			mp3_current = main_data->track[ mp3_cur_track ].length * mp3_progress;
-
-			switch ( temp ) {
-			case PLUGIN_MSG_PARSE_ERR :
-				/* Nothing to do. Let's wait more */
-				break;
-
-			case PLUGIN_PROGRESS_MSG :
-				/* progress report, update status display */
-				calc_stat( main_data, &stat, mp3_current,
-				           mp3_cur_track, MP3, CALC_UPDATE );
-				/* Update status widget */
-				if ( msg[ 0 ] == '\0' )
-					str = NULL;
-				else
-					str = msg;
-				wm_status_frame_handler( WIDGET_UPDATE, MP3, &stat, str );
-				break;
-
-			case PLUGIN_NO_MSG_AVAILABLE :
-				/* Check if the plugin has exited
-				 It only happens when ripperX failed to execute the plugin */
-				if ( mp3_pi_pid >= 0 )
-					if ( waitpid( mp3_pi_pid, NULL, WNOHANG ) == mp3_pi_pid ) {
-						err_handler( PLUGIN_NOT_PRESENT_ERR,
-						             "Maybe ripperX has failed to execute the plugin" );
-						mp3_pi_pid = -1;
-						job_controller_timeout_stop();
-						job_controller( JC_ABORT_ALL_DELETE, main_data );
-						return;
-					}
-
-				/* Check if the job is finished */
-				if ( waitpid( mp3_pg_pid, NULL, WNOHANG ) == mp3_pg_pid ) {
-					/* One job finished, go for next one */
-
-					/* kill the plugin */
-					if ( waitpid( mp3_pi_pid, NULL, WNOHANG ) != mp3_pi_pid ) {
-						kill( mp3_pi_pid, SIGTERM );
-						waitpid( mp3_pi_pid, NULL, 0 );
-					}
-
-					/* Close the fd */
-					close( mp3_read_fd );
-
-					/* stop calculating stats */
-					calc_stat( main_data, &stat, mp3_current,
-					           mp3_cur_track, MP3, CALC_STOP );
-
-					main_data->track[ mp3_cur_track ].mp3_exist = TRUE;
-					/* Delete WAV file if he/she doesn't want it */
-					if ( !config.keep_wav ) {
-					create_file_names_for_track(main_data, mp3_cur_track,
-												&wav_file_path, &enc_file_path);
-						if ( unlink( wav_file_path ) < 0 )
-							err_handler( FILE_DELETE_ERR, wav_file_path );
-						/* Mark that it has been deleted */
-						main_data->track[ mp3_cur_track ].wav_exist = FALSE;
-						
-						/* Delete WAV work directory if this was the last WAV file,
-						and the mp3 work dir is different */
-						if (stat.tracks_remain <= 1)
-							if (strcmp(config.wav_path,config.mp3_path) != 0)
-						 		rmdir(file_path_without_name(wav_file_path));
-					}
-
-					/* find next track to encode */
-					temp_track = mp3_cur_track;
-					if ( find_next_job( main_data, mp3_cur_track, MP3, &mp3_nxt_track, &cur_type ) < 0 ) {
-						/* All finished - no more encoding - done for good! */
-						mp3_pg_pid = -1;
-						mp3_pi_pid = -1;
-						mp3_cur_track = mp3_nxt_track = -1;
-						cur_type = -1;
-						calc_stat( main_data, &stat, mp3_current,
-						           temp_track, MP3, CALC_STOP_SESSION );
-						job_controller_timeout_stop();
-						stat.encoding = FALSE;
-
-						job_finisher( main_data );
-						return;
-					}
-					mp3_cur_track = -1;
-
-					stat.tracks_done++;
-					stat.tracks_remain--;
-					stat.encoding = FALSE;
-
-				} /* end if job is finished section */
-				break;
-			} /* end encode switch */
-		} /* end encode progress check */
-
-		if ( !stat.encoding			&& 
-			 mp3_nxt_track != -1	&&
-			 main_data->track[ mp3_nxt_track ].wav_exist)
-		{
-
-			/* start encoding */
-			mp3_cur_track = mp3_nxt_track;
-			mp3_nxt_track = -1;
-			create_file_names_for_track(main_data, mp3_cur_track,
-										&wav_file_path, &enc_file_path);
-			calc_stat( main_data, &stat, mp3_current,
-					   mp3_cur_track, MP3, CALC_START );
-			if ( start_ripping_encoding( MP3,
-											 main_data->track[ mp3_cur_track ].begin,
-											 main_data->track[ mp3_cur_track ].length,
-											 mp3_cur_track,
-											 wav_file_path,
-											 enc_file_path,
-											 &mp3_pg_pid, &mp3_pi_pid,
-											 &mp3_read_fd ) < 0 )
+			if ( !stat.encoding && mp3_nxt_track != -1 && main_data->track[ mp3_nxt_track ].wav_exist)
 			{
-				calc_stat( main_data, &stat, mp3_current,
-						   mp3_cur_track, MP3, CALC_STOP_SESSION );
-				job_controller_timeout_stop();
-				job_finisher( main_data );
-			}
-			stat.encoding = TRUE;
-		}
-
-		/* end of JC_UPDATE */
-		return;
-	case JC_PAUSE :
-		main_data = saved_main_data;
-
-		if ( wav_pg_pid >= 0 )
-			if ( waitpid( wav_pg_pid, NULL, WNOHANG ) != wav_pg_pid )
-				kill( wav_pg_pid, SIGTSTP );
-		if ( wav_pi_pid >= 0 )
-			if ( waitpid( wav_pi_pid, NULL, WNOHANG ) != wav_pi_pid )
-				kill( wav_pi_pid, SIGTSTP );
-		if ( mp3_pg_pid >= 0 )
-			if ( waitpid( mp3_pg_pid, NULL, WNOHANG ) != mp3_pg_pid )
-				kill( mp3_pg_pid, SIGTSTP );
-		if ( mp3_pi_pid >= 0 )
-			if ( waitpid( mp3_pi_pid, NULL, WNOHANG ) != mp3_pi_pid )
-				kill( mp3_pi_pid, SIGTSTP );
-
-		calc_stat( main_data, &stat, wav_current,
-		           wav_cur_track, WAV, CALC_PAUSE );
-		job_controller_timeout_stop();
-		return;
-
-	case JC_CONT :
-		main_data = saved_main_data;
-
-		if ( wav_pg_pid >= 0 )
-			if ( waitpid( wav_pg_pid, NULL, WNOHANG ) != wav_pg_pid )
-				kill( wav_pg_pid, SIGCONT );
-		if ( wav_pi_pid >= 0 )
-			if ( waitpid( wav_pi_pid, NULL, WNOHANG ) != wav_pi_pid )
-				kill( wav_pi_pid, SIGCONT );
-		if ( mp3_pg_pid >= 0 )
-			if ( waitpid( mp3_pg_pid, NULL, WNOHANG ) != mp3_pg_pid )
-				kill( mp3_pg_pid, SIGCONT );
-		if ( mp3_pi_pid >= 0 )
-			if ( waitpid( mp3_pi_pid, NULL, WNOHANG ) != mp3_pi_pid )
-				kill( mp3_pi_pid, SIGCONT );
-
-		calc_stat( main_data, &stat, wav_current,
-		           wav_cur_track, cur_type, CALC_CONT );
-		job_controller_timeout_start();
-		return;
-
-	case JC_ABORT :
-	case JC_ABORT_DELETE :
-	case JC_ABORT_ALL :
-	case JC_ABORT_ALL_DELETE :
-		main_data = saved_main_data;
-
-		if ( wav_pg_pid >= 0 )
-			if ( waitpid( wav_pg_pid, NULL, WNOHANG ) != wav_pg_pid ) {
-				job_controller( JC_CONT, NULL );
-				kill( wav_pg_pid, SIGINT );
-				waitpid( wav_pg_pid, NULL, 0 );
-			}
-		if ( wav_pi_pid >= 0 )
-			if ( waitpid( wav_pi_pid, NULL, WNOHANG ) != wav_pi_pid ) {
-				kill( wav_pi_pid, SIGINT );
-				waitpid( wav_pi_pid, NULL, 0 );
-			}
-		if ( mp3_pg_pid >= 0 )
-			if ( waitpid( mp3_pg_pid, NULL, WNOHANG ) != mp3_pg_pid ) {
-				job_controller( JC_CONT, NULL );
-				kill( mp3_pg_pid, SIGINT );
-				waitpid( mp3_pg_pid, NULL, 0 );
-			}
-		if ( mp3_pi_pid >= 0 )
-			if ( waitpid( mp3_pi_pid, NULL, WNOHANG ) != mp3_pi_pid ) {
-				kill( mp3_pi_pid, SIGINT );
-				waitpid( mp3_pi_pid, NULL, 0 );
+				/* start encoding */
+				mp3_cur_track = mp3_nxt_track;
+				mp3_nxt_track = -1;
+				create_file_names_for_track(main_data, mp3_cur_track, &wav_file_path, &enc_file_path);
+				calc_stat( main_data, &stat, mp3_current, mp3_cur_track, MP3, CALC_START );
+				if ( start_ripping_encoding( MP3, main_data->track[ mp3_cur_track ].begin,
+							main_data->track[ mp3_cur_track ].length,
+							mp3_cur_track,
+							wav_file_path,
+							enc_file_path,
+							&mp3_pg_pid, &mp3_pi_pid,
+							&mp3_read_fd ) < 0 )
+				{
+					calc_stat( main_data, &stat, mp3_current, mp3_cur_track, MP3, CALC_STOP_SESSION );
+					job_controller_timeout_stop();
+					job_finisher( main_data );
+				}
+				stat.encoding = TRUE;
 			}
 
-		/* Close the pipe or pty */
-		close( wav_read_fd );
-		close( mp3_read_fd );
-		calc_stat( main_data, &stat, wav_current,
-		           wav_cur_track, cur_type, CALC_STOP_SESSION );
+			/* end of JC_UPDATE */
+			return;
+		case JC_PAUSE :
+			main_data = saved_main_data;
 
-		/* Destroy status widget */
-		wm_status_frame_handler( WIDGET_DESTROY, WAV, &stat, msg );
+			if ( wav_pg_pid >= 0 )
+				if ( waitpid( wav_pg_pid, NULL, WNOHANG ) != wav_pg_pid )
+					kill( wav_pg_pid, SIGTSTP );
+			if ( wav_pi_pid >= 0 )
+				if ( waitpid( wav_pi_pid, NULL, WNOHANG ) != wav_pi_pid )
+					kill( wav_pi_pid, SIGTSTP );
+			if ( mp3_pg_pid >= 0 )
+				if ( waitpid( mp3_pg_pid, NULL, WNOHANG ) != mp3_pg_pid )
+					kill( mp3_pg_pid, SIGTSTP );
+			if ( mp3_pi_pid >= 0 )
+				if ( waitpid( mp3_pi_pid, NULL, WNOHANG ) != mp3_pi_pid )
+					kill( mp3_pi_pid, SIGTSTP );
 
-		if ( ( ops == JC_ABORT_ALL_DELETE ) || ( ops == JC_ABORT_DELETE ) ) {
-			create_file_names_for_track(main_data, wav_cur_track,
-										&wav_file_path, NULL);
-			if ( wav_cur_track != -1 ) {
-				create_file_names_for_track(main_data, wav_cur_track,
-											&wav_file_path, &enc_file_path);
-				if ( unlink( wav_file_path ) < 0 )
-					err_handler( FILE_DELETE_ERR, wav_file_path );
+			calc_stat( main_data, &stat, wav_current, wav_cur_track, WAV, CALC_PAUSE );
+			job_controller_timeout_stop();
+			return;
+
+		case JC_CONT :
+			main_data = saved_main_data;
+
+			if ( wav_pg_pid >= 0 )
+				if ( waitpid( wav_pg_pid, NULL, WNOHANG ) != wav_pg_pid )
+					kill( wav_pg_pid, SIGCONT );
+			if ( wav_pi_pid >= 0 )
+				if ( waitpid( wav_pi_pid, NULL, WNOHANG ) != wav_pi_pid )
+					kill( wav_pi_pid, SIGCONT );
+			if ( mp3_pg_pid >= 0 )
+				if ( waitpid( mp3_pg_pid, NULL, WNOHANG ) != mp3_pg_pid )
+					kill( mp3_pg_pid, SIGCONT );
+			if ( mp3_pi_pid >= 0 )
+				if ( waitpid( mp3_pi_pid, NULL, WNOHANG ) != mp3_pi_pid )
+					kill( mp3_pi_pid, SIGCONT );
+
+			calc_stat( main_data, &stat, wav_current, wav_cur_track, cur_type, CALC_CONT );
+			job_controller_timeout_start();
+			return;
+
+		case JC_ABORT :
+		case JC_ABORT_DELETE :
+		case JC_ABORT_ALL :
+		case JC_ABORT_ALL_DELETE :
+			main_data = saved_main_data;
+
+			if ( wav_pg_pid >= 0 )
+				if ( waitpid( wav_pg_pid, NULL, WNOHANG ) != wav_pg_pid ) {
+					job_controller( JC_CONT, NULL );
+					kill( wav_pg_pid, SIGINT );
+					waitpid( wav_pg_pid, NULL, 0 );
+				}
+			if ( wav_pi_pid >= 0 )
+				if ( waitpid( wav_pi_pid, NULL, WNOHANG ) != wav_pi_pid ) {
+					kill( wav_pi_pid, SIGINT );
+					waitpid( wav_pi_pid, NULL, 0 );
+				}
+			if ( mp3_pg_pid >= 0 )
+				if ( waitpid( mp3_pg_pid, NULL, WNOHANG ) != mp3_pg_pid ) {
+					job_controller( JC_CONT, NULL );
+					kill( mp3_pg_pid, SIGINT );
+					waitpid( mp3_pg_pid, NULL, 0 );
+				}
+			if ( mp3_pi_pid >= 0 )
+				if ( waitpid( mp3_pi_pid, NULL, WNOHANG ) != mp3_pi_pid ) {
+					kill( mp3_pi_pid, SIGINT );
+					waitpid( mp3_pi_pid, NULL, 0 );
+				}
+
+			/* Close the pipe or pty */
+			close( wav_read_fd );
+			close( mp3_read_fd );
+			calc_stat( main_data, &stat, wav_current, wav_cur_track, cur_type, CALC_STOP_SESSION );
+
+			/* Destroy status widget */
+			wm_status_frame_handler( WIDGET_DESTROY, WAV, &stat, msg );
+
+			if ( ( ops == JC_ABORT_ALL_DELETE ) || ( ops == JC_ABORT_DELETE ) ) {
+				create_file_names_for_track(main_data, wav_cur_track, &wav_file_path, NULL);
+				if ( wav_cur_track != -1 ) {
+					create_file_names_for_track(main_data, wav_cur_track, &wav_file_path, &enc_file_path);
+					if ( unlink( wav_file_path ) < 0 )
+						err_handler( FILE_DELETE_ERR, wav_file_path );
+				}
+				if ( mp3_cur_track != -1 ) {
+					create_file_names_for_track(main_data, mp3_cur_track, NULL, &enc_file_path);
+					if ( unlink( enc_file_path ) < 0 )
+						err_handler( FILE_DELETE_ERR, enc_file_path );
+				}
+			} else {
+				/* Mark that it exists */
+				if ( wav_cur_track != -1 )
+					main_data->track[ wav_cur_track ].wav_exist = TRUE;
+				if ( mp3_cur_track != -1 )
+					main_data->track[ mp3_cur_track ].mp3_exist = TRUE;
 			}
-			if ( mp3_cur_track != -1 ) {
-				create_file_names_for_track(main_data, mp3_cur_track,
-											NULL, &enc_file_path);
-				if ( unlink( enc_file_path ) < 0 )
-					err_handler( FILE_DELETE_ERR, enc_file_path );
-			}
-		} else {
-			/* Mark that it exists */
-			if ( wav_cur_track != -1 )
-				main_data->track[ wav_cur_track ].wav_exist = TRUE;
-			if ( mp3_cur_track != -1 )
-				main_data->track[ mp3_cur_track ].mp3_exist = TRUE;
-		}
 
-		wav_cur_track = -1;
-		mp3_cur_track = -1;
-		cur_type = -1;
-		calc_stat( main_data, &stat, wav_current,
-		           wav_cur_track, cur_type, CALC_STOP_SESSION );
-		job_controller_timeout_stop();
-		job_finisher( main_data );
-		return;
+			wav_cur_track = -1;
+			mp3_cur_track = -1;
+			cur_type = -1;
+			calc_stat( main_data, &stat, wav_current,
+					wav_cur_track, cur_type, CALC_STOP_SESSION );
+			job_controller_timeout_stop();
+			job_finisher( main_data );
+			return;
 	}
 }
