@@ -47,15 +47,15 @@ void sf_album_entry_changed( GtkWidget *widget, gpointer callback_data )
 
 // patch from M.Tyler
 // tm:  we're not quite read for this (need updates to cddb.c)
-//void sf_year_entry_changed( GtkWidget *widget, gpointer callback_data )
-//{
-//	select_frame_handler( SF_YEAR_ENTRY_CHG, 0, NULL );
-//}
-//
-//void sf_genre_entry_changed( GtkWidget *widget, gpointer callback_data )
-//{
-//	select_frame_handler( SF_GENRE_ENTRY_CHG, 0, NULL );
-//}
+void sf_year_entry_changed( GtkWidget *widget, gpointer callback_data )
+{
+	select_frame_handler( SF_YEAR_ENTRY_CHG, 0, NULL );
+}
+
+void sf_genre_entry_changed( GtkWidget *widget, gpointer callback_data )
+{
+	select_frame_handler( SF_GENRE_ENTRY_CHG, 0, NULL );
+}
 
 void sf_filename_entry_changed( GtkWidget *widget, gpointer callback_data )
 {
@@ -96,10 +96,10 @@ void select_frame_handler( int ops, int track, _main_data *main_data )
 	static GtkWidget *album_label = NULL;
 	static GtkWidget *album_entry = NULL;
 
-	//static GtkWidget *year_label = NULL;
-	//static GtkWidget *year_entry = NULL;
-	//static GtkWidget *genre_label = NULL;
-	//static GtkWidget *genre_entry = NULL;
+	static GtkWidget *year_label = NULL;
+	static GtkWidget *year_entry = NULL;
+	static GtkWidget *genre_label = NULL;
+	static GtkWidget *genre_entry = NULL;
 	//static GtkWidget *genre_combo = NULL;
 	//static GList *genre_combo_items = NULL;
 
@@ -114,7 +114,7 @@ void select_frame_handler( int ops, int track, _main_data *main_data )
 
 	switch ( ops ) {
 	case WIDGET_CREATE : {
-			GtkWidget * main_frame, *vbox, *hbox1, *vbox2, *hbox, *hbox2, *hbox3, *label, *scr_window;
+			GtkWidget * main_frame, *vbox, *hbox1, *vbox2, *hbox, *hbox2, *hbox3, *hbox4, *hbox5, *hbox6, *label, *scr_window;
 			GtkWidget *table, *separator, *button;
 			GtkStyle *style;
 			GdkGC *gc;
@@ -214,10 +214,10 @@ void select_frame_handler( int ops, int track, _main_data *main_data )
 			gtk_container_border_width( GTK_CONTAINER( vbox2 ), 0 );
 			gtk_scrolled_window_add_with_viewport( GTK_SCROLLED_WINDOW( scr_window ), vbox2 );
 
-			/* artist and album entry fields */
+			/* artist entry fields */
 			hbox3 = gtk_hbox_new (FALSE, 0);
 
-			artist_label = gtk_label_new("Artist");
+			artist_label = gtk_label_new("Artist ");
 			gtk_box_pack_start(GTK_BOX(hbox3), artist_label, FALSE, FALSE, 5);
 			artist_entry = gtk_entry_new_with_max_length(MAX_ARTIST_LENGTH);
 			gtk_entry_set_text(GTK_ENTRY(artist_entry), main_data->disc_artist);
@@ -225,30 +225,45 @@ void select_frame_handler( int ops, int track, _main_data *main_data )
 			                    GTK_SIGNAL_FUNC( sf_artist_entry_changed ),
 			                    NULL );
 			gtk_box_pack_start(GTK_BOX(hbox3), artist_entry, TRUE, TRUE, 5);
+			gtk_box_pack_start(GTK_BOX(vbox2), hbox3, FALSE, FALSE, 5);
 
+			/* album entry fields */
+			hbox4 = gtk_hbox_new (FALSE, 0);
 			album_label = gtk_label_new("Album");
-			gtk_box_pack_start(GTK_BOX(hbox3), album_label, FALSE, FALSE, 5);
+			gtk_box_pack_start(GTK_BOX(hbox4), album_label, FALSE, FALSE, 5);
 			album_entry = gtk_entry_new_with_max_length(MAX_ARTIST_LENGTH);
 			gtk_entry_set_text(GTK_ENTRY(album_entry), main_data->disc_title);
 			gtk_signal_connect( GTK_OBJECT( album_entry ), "changed",
 			                    GTK_SIGNAL_FUNC( sf_album_entry_changed ),
 			                    NULL );
-			gtk_box_pack_start(GTK_BOX(hbox3), album_entry, TRUE, TRUE, 5);
+			gtk_box_pack_start(GTK_BOX(hbox4), album_entry, TRUE, TRUE, 5);
+			gtk_box_pack_start(GTK_BOX(vbox2), hbox4, FALSE, FALSE, 5);
+
+			/* year and genre entry fields */
+			hbox5 = gtk_hbox_new (FALSE, 0);
+
+			year_label = gtk_label_new("Year");
+			gtk_box_pack_start(GTK_BOX(hbox5), year_label, FALSE, FALSE, 5);
+			year_entry = gtk_entry_new_with_max_length(MAX_YEAR_LENGTH);
+			gtk_widget_set_usize(year_entry, 60, -2);
+			if ( strlen(main_data->disc_year) < 2 )
+				sprintf(main_data->disc_year, "2005");
+			gtk_entry_set_text(GTK_ENTRY(year_entry), main_data->disc_year);
+			gtk_signal_connect( GTK_OBJECT( year_entry ), "changed",
+			                    GTK_SIGNAL_FUNC( sf_year_entry_changed ),
+			                    NULL );
+			gtk_box_pack_start(GTK_BOX(hbox5), year_entry, TRUE, TRUE, 5);
+
+			genre_label = gtk_label_new("Genre");
+			gtk_box_pack_start(GTK_BOX(hbox5), genre_label, FALSE, FALSE, 5);
+			genre_entry = gtk_entry_new_with_max_length(MAX_GENRE_LENGTH);
+			gtk_entry_set_text(GTK_ENTRY(genre_entry), main_data->disc_category);
+			gtk_signal_connect( GTK_OBJECT( genre_entry ), "changed",
+					    GTK_SIGNAL_FUNC( sf_genre_entry_changed ),
+					    NULL );
+			gtk_box_pack_start(GTK_BOX(hbox5), genre_entry, TRUE, TRUE, 5);
+			gtk_box_pack_start(GTK_BOX(vbox2), hbox5, FALSE, FALSE, 5);
 
-//			year_label = gtk_label_new("Year");
-//			gtk_box_pack_start(GTK_BOX(hbox3), year_label, FALSE, FALSE, 5);
-//			year_entry = gtk_entry_new_with_max_length(MAX_YEAR_LENGTH);
-//			gtk_widget_set_usize(year_entry, 60, -2);
-//			if ( strlen(main_data->disc_year) < 2 )
-//				sprintf(main_data->disc_year, "2005");
-//			gtk_entry_set_text(GTK_ENTRY(year_entry), main_data->disc_year);
-//			gtk_signal_connect( GTK_OBJECT( year_entry ), "changed",
-//			                    GTK_SIGNAL_FUNC( sf_year_entry_changed ),
-//			                    NULL );
-//			gtk_box_pack_start(GTK_BOX(hbox3), year_entry, TRUE, TRUE, 5);
-//
-//			genre_label = gtk_label_new("Genre");
-//			gtk_box_pack_start(GTK_BOX(hbox3), genre_label, FALSE, FALSE, 5);
 //			genre_combo = gtk_combo_new ();
 //			gtk_widget_show (genre_combo);
 //			gtk_box_pack_start(GTK_BOX(hbox3), genre_combo, TRUE, TRUE, 5);
@@ -256,7 +271,7 @@ void select_frame_handler( int ops, int track, _main_data *main_data )
 //			gtk_container_set_border_width (GTK_CONTAINER (genre_combo), 5);
 //			gtk_combo_set_value_in_list (GTK_COMBO (genre_combo), TRUE, FALSE);
 //			gtk_combo_set_use_arrows_always (GTK_COMBO (genre_combo), TRUE);
-//
+
 //			i = 0;
 //			while ( id3_styles[i].name != NULL )
 //			{
@@ -280,7 +295,6 @@ void select_frame_handler( int ops, int track, _main_data *main_data )
 //			gtk_widget_show (genre_entry);
 
 
-			gtk_box_pack_start(GTK_BOX(vbox2), hbox3, FALSE, FALSE, 5);
 			
 			/* the table */
 			table = gtk_table_new( MAX_NUM_TRACK, 6, FALSE );
@@ -404,17 +418,17 @@ void select_frame_handler( int ops, int track, _main_data *main_data )
 		        gtk_entry_get_text( GTK_ENTRY( album_entry ) ) );
 		return;
 
-//	case SF_YEAR_ENTRY_CHG :
-//		/* The album year has been edited. Sync saved album year */
-//		strcpy( saved_main_data->disc_year,
-//		        gtk_entry_get_text( GTK_ENTRY( year_entry ) ) );
-//		return;
-//
-//	case SF_GENRE_ENTRY_CHG :
-//		/* The album genre has been edited. Sync saved album genre */
-//		strcpy( saved_main_data->disc_category,
-//		        gtk_entry_get_text( GTK_ENTRY( genre_entry ) ) );
-//		return;
+	case SF_YEAR_ENTRY_CHG :
+		/* The album year has been edited. Sync saved album year */
+		strcpy( saved_main_data->disc_year,
+		        gtk_entry_get_text( GTK_ENTRY( year_entry ) ) );
+		return;
+
+	case SF_GENRE_ENTRY_CHG :
+		/* The album genre has been edited. Sync saved album genre */
+		strcpy( saved_main_data->disc_category,
+		        gtk_entry_get_text( GTK_ENTRY( genre_entry ) ) );
+		return;
 
 	case SF_FILENAME_ENTRY_CHG :
 		if ( strlen( gtk_entry_get_text( GTK_ENTRY( filename_entry[ track ] ) ) ) == 0 ) {
@@ -430,9 +444,15 @@ void select_frame_handler( int ops, int track, _main_data *main_data )
 			/* The file name has been edited. Sync saved file name */
 			strcpy( saved_filename_entry[ track ],
 			        gtk_entry_get_text( GTK_ENTRY( filename_entry[ track ] ) ) );
+			/* 
+			 * 20051030/tm
+			 * no longer auto-select the track when the text is altered
+			 * 
+			 
 			if ( GTK_TOGGLE_BUTTON( track_selected_button[ track ] ) ->active == FALSE )
 				gtk_toggle_button_set_state( GTK_TOGGLE_BUTTON( track_selected_button[ track ] ),
 				                             TRUE );
+			*/
 		}
 		return;
 
@@ -478,6 +498,8 @@ void select_frame_handler( int ops, int track, _main_data *main_data )
 
 		gtk_entry_set_text(GTK_ENTRY(artist_entry), main_data->disc_artist);
 		gtk_entry_set_text(GTK_ENTRY(album_entry), main_data->disc_title);
+		gtk_entry_set_text(GTK_ENTRY(year_entry), main_data->disc_year);
+		gtk_entry_set_text(GTK_ENTRY(genre_entry), main_data->disc_category);
 
 		for ( track = 0; track < num_tracks; track++ ) {
 			button_state = GTK_TOGGLE_BUTTON( track_selected_button[ track ] ) ->active;
